NUM1     EQU    20H      ; 定义数据存储单元 - 千位数值存储地址
NUM2     EQU    21H      ; 百位数值存储地址
NUM3     EQU    22H      ; 十位数值存储地址（键盘输入暂存）
NUM4     EQU    23H      ; 个位数值存储地址（键盘输入暂存）
ZI1      EQU    24H      ; 字形码存储单元 - 千位数码管显示码
ZI2      EQU    25H      ; 百位数码管显示码
ZI3      EQU    26H      ; 十位数码管显示码（键盘输入对应）
ZI4      EQU    27H      ; 个位数码管显示码（键盘输入对应）
BIAO     EQU    40H      ; 标志位存储单元 - 控制键盘输入对应位（0:个位 1:十位）
DAT      BIT    P2.0     ; ADC数据传输引脚定义
CLK      BIT    P2.1     ; ADC时钟控制引脚
CS       BIT    P2.2     ; ADC片选控制引脚

        ORG    0000H       ; 程序存储器起始地址
        LJMP   START       ; 跳转至主程序入口

        ORG    0003H       ; 外部中断0（INT0）入口地址
        LJMP   INT_0       ; 跳转至外部中断处理程序

        ORG    000BH       ; 定时器0中断入口地址
        LJMP   T0_INT      ; 跳转至定时器中断处理程序

        ORG    0030H       ; 主程序起始地址

START:  
        MOV    SP, #60H    ; 设置堆栈指针（SP）到0x60，预留中断保护空间
        MOV    TCON, #00H   ; 初始化中断控制寄存器：
                            ; IT0=0（INT0电平触发），IE0=0（清除中断标志）
        SETB   EA          ; 开启总中断允许（CPU响应中断）
        SETB   EX0         ; 开启外部中断0允许
        MOV    TMOD, #01H  ; 设置定时器0为模式1（16位自动重装）
        MOV    TL0, #033H   ; 定时器0低8位初值：0xFD33（0.5ms定时）
        MOV    TH0, #0FEH   ; 定时器0高8位初值（晶振12MHz时的计算值）
        SETB   TR0         ; 启动定时器0
        SETB   ET0         ; 开启定时器0中断允许

        SETB   BIAO        ; 初始化标志位为1（默认先操作十位数码管）
        MOV    ZI1, #0C0H  ; 初始化千位显示为0（0C0H是数码管0的字形码）
        MOV    ZI2, #0C0H  ; 百位显示初始化为0
        MOV    ZI3, #0C0H  ; 十位显示初始化为0
        MOV    ZI4, #0C0H  ; 个位显示初始化为0

        SETB   DAT         ; ADC初始化：DAT引脚先置高（作为输入模式准备）
LOOP3:  
        LCALL   TCL549_ADC  ; 调用ADC转换函数，获取模拟量数据
        LCALL   CHANGE      ; 数据处理：将ADC值转换为显示用的字形码
        MOV    P1, #0FFH    ; 关闭所有数码管位选（熄灭所有）
        
        ; 千位数码管显示
        MOV    P0, ZI1      ; 发送千位字形码到段选端口（P0口）
        MOV    P1, #0EFH    ; 位选控制：P1.4=0（千位数码管使能，共阴结构）
        LCALL   DELAY       ; 调用延时函数，保持显示稳定
        
        ; 百位数码管显示
        MOV    P1, #0FFH    ; 关闭其他位选
        MOV    P0, ZI2      ; 发送百位字形码
        MOV    P1, #0DFH    ; 位选控制：P1.5=0（百位数码管使能）
        LCALL   DELAY
        
        ; 十位数码管显示（键盘输入位1）
        MOV    P1, #0FFH
        MOV    P0, ZI3      ; 发送十位字形码
        MOV    P1, #0BFH    ; 位选控制：P1.6=0（十位数码管使能）
        LCALL   DELAY
        
        ; 个位数码管显示（键盘输入位2）
        MOV    P1, #0FFH
        MOV    P0, ZI4      ; 发送个位字形码
        MOV    P1, #07FH    ; 位选控制：P1.7=0（个位数码管使能）
        LCALL   DELAY
        
        SJMP    LOOP3       ; 循环执行显示流程

; 数码管字形码表（共阴极，0-15对应的段码）
LEDGY:   DB     0C0H, 0F9H, 0A4H, 0B0H, 99H, 92H, 82H, 0F8H  ; 0-7
        DB     80H, 90H, 88H, 83H, 0C6H, 0A1H, 86H, 8EH       ; 8-F

; 键盘键值表（P1口扫描码，对应16个按键的物理键值）
KEYTAB:  DB     077H, 07BH, 07DH, 07EH, 0B7H, 0BBH, 0BDH     ; 第一行按键
        DB     0BEH, 0D7H, 0DBH, 0DDH, 0DEH, 0E7H, 0EBH     ; 第二行按键
        DB     0EDH, 0EEH                                       ; 第三行按键（假设矩阵键盘为4x4？此处定义15个，可能需检查）

; 外部中断0处理程序（矩阵键盘输入检测）
INT_0:  
        PUSH   PSW         ; 保护现场：压栈保存程序状态字
        PUSH   00H         ; 压栈保存通用寄存器R0
        PUSH   01H         ; 压栈保存通用寄存器R1
        PUSH   ACC         ; 压栈保存累加器ACC
        
        LCALL   DELAY       ; 消抖延时（去除按键抖动干扰）
        MOV    DPTR, #KEYTAB; 加载键盘键值表地址到数据指针
        MOV    R1, #00H    ; 键值索引寄存器初始化（从第一个键开始查找）

JUMP2:  
        MOV    A, R1       ; 取当前索引值
        MOVC   A, @A+DPTR   ; 查表获取键值（A = KEYTAB[R1]）
        CJNE   A, P1, JUMP1 ; 比较P1口当前值与键值：不等则查找下一个键
        SJMP   JUMP3       ; 相等则跳转处理按键值

JUMP1:  
        INC    R1          ; 键值索引递增
        SJMP   JUMP2       ; 继续查找下一个键

JUMP3:  
; 按键处理：将物理键值转换为逻辑数值并更新对应显示位
        MOV    DPTR, #LEDGY; 加载字形码表地址
        MOV    A, R1       ; 取按键索引（0-15对应0-F）
        MOVC   A, @A+DPTR   ; 查表获取对应字形码
        
        JNB    BIAO, SECOND; 检查标志位：BIAO=0时操作个位，否则操作十位
        ; 标志位为1时处理十位（NUM3/ZI3）
        MOV    NUM3, R1    ; 保存十位数值索引（0-15）
        MOV    ZI3, A      ; 更新十位数码管显示码
        SJMP   NEXT        ; 跳过个位处理

SECOND:  
        ; 标志位为0时处理个位（NUM4/ZI4）
        MOV    NUM4, R1    ; 保存个位数值索引
        MOV    ZI4, A      ; 更新个位数码管显示码

NEXT:   
        CPL    BIAO        ; 翻转标志位，下次切换处理位（0↔1）
        LCALL   DELAY       ; 消抖延时
        JNB    P3.2, $     ; 等待按键释放（P3.2是INT0引脚，低电平触发）
        MOV    P1, #0FFH    ; 清空P1口输入状态
        
        POP    ACC         ; 恢复现场：按压栈顺序弹出寄存器
        POP    01H
        POP    00H
        POP    PSW
        RETI                ; 中断返回

; TCL549 ADC转换函数（假设使用TCL549型ADC）
TCL549_ADC:  
        PUSH   07H         ; 保护寄存器R7
        CLR    A           ; 初始化累加器
        SETB   CS          ; 片选信号初始化为高（禁止ADC）
        CLR    CLK         ; 时钟信号初始化为低
        
        MOV    R7, #08H    ; 设定8位数据传输计数
        CLR    CS          ; 拉低片选信号，选中ADC
        NOP                ; 空操作（等待信号稳定）

LOOP1:  
        SETB   CLK         ; 时钟上升沿：读取DAT引脚数据
        MOV    C, DAT      ; 将DAT引脚状态存入进位标志C
        RLC    A           ; 带进位左移：将数据移入累加器（最高位先读）
        CLR    CLK         ; 时钟下降沿：准备下一次数据传输
        DJNZ   R7, LOOP1   ; 循环8次完成一个字节读取
        
        SETB   CS          ; 拉高片选信号，结束转换
        CLR    CLK         ; 复位时钟信号
        SETB   DAT         ; 恢复DAT引脚为输入模式（上拉）
        POP    07H         ; 恢复寄存器R7
        RET                ; 函数返回（结果在A中）

; 数据处理函数：将ADC值转换为十进制并生成字形码
CHANGE:   
        MOV    30H, A      ; 暂存ADC转换结果到30H单元
        MOV    B, #100     ; 计算百位数值：除以100
        DIV    AB          ; 除法运算：A=商（百位），B=余数
        MOV    A, B        ; 取余数继续处理十位和个位
        MOV    31H, B      ; 暂存余数（用于蜂鸣器比较）
        
        MOV    B, #10      ; 计算十位数值：余数除以10
        DIV    AB          ; 除法运算：A=商（十位），B=余数（个位）
        MOV    NUM1, A     ; 保存十位数值到NUM1（千位显示？此处可能需注意变量名对应）
        MOV    NUM2, B     ; 保存个位数值到NUM2（百位显示？）
        
        MOV    R0, #NUM1   ; 初始化数据指针：指向十位数值存储地址
        MOV    R1, #ZI1    ; 初始化字形码指针：指向千位显示存储地址
        MOV    R2, #2      ; 循环次数：处理两个高位（千位+百位）

LOOP2:  
        MOV    A, @R0      ; 取当前数值（十位/个位原始值）
        MOV    DPTR, #LEDGY; 加载字形码表地址
        MOVC   A, @A+DPTR   ; 查表获取对应字形码
        MOV    @R1, A      ; 保存字形码到对应显示单元
        INC    R0          ; 数据指针指向下一个数值存储地址
        INC    R1          ; 字形码指针指向下一个显示单元
        DJNZ   R2, LOOP2   ; 循环处理两个高位

EXIT1:  
        RET                ; 函数返回

; 定时器0中断处理程序（蜂鸣器控制）
T0_INT:  
        PUSH   PSW         ; 保护现场
        PUSH   ACC
        PUSH   00H         ; 保护R0寄存器
        
        MOV    TL0, #033H  ; 重新加载定时器初值（0.5ms定时）
        MOV    TH0, #0FEH
        
        MOV    A, NUM3     ; 取十位输入值（R1存储的是按键索引，需确认是否正确转换）
        MOV    B, #10      ; 计算十位数值实际值：索引×10
        MUL    AB          ; 乘法运算：A=十位数值×10
        MOV    R0, NUM4    ; 取个位输入值（直接使用存储的索引？需确认逻辑）
        ADD    A, R0       ; 计算输入的两位数总和：十位×10 + 个位
        
        MOV    PSW, #00H   ; 清除PSW标志位（确保比较操作准确）
        CJNE   A, 31H, NEXT1; 比较输入值与ADC转换值：不相等则跳转
        ; 相等时不做处理（蜂鸣器状态不变）

NEXT1:  
        JNC    EXIT        ; 无进位（A≥31H）时退出，否则翻转蜂鸣器引脚
        CPL    P2.3        ; 翻转蜂鸣器控制引脚（P2.3），产生声音

EXIT:   
        POP    00H         ; 恢复现场
        POP    ACC
        POP    PSW
        RETI                ; 中断返回

; 延时函数（约1ms@12MHz晶振，实际延时需根据硬件调整）
DELAY:  
        PUSH   00H         ; 保护R0
        PUSH   01H         ; 保护R1
        MOV    R1, #00H    ; 外层循环计数

LOOP4:  
        MOV    R0, #06H    ; 内层循环计数
        DJNZ   R0, $       ; 空操作循环（R0自减到0）
        DJNZ   R1, LOOP4   ; 外层循环
        POP    01H         ; 恢复寄存器
        POP    00H
        RET                ; 函数返回

        END                ; 程序结束